@@ -119,10 +119,7 @@ struct module *find_symbol(const char *name, const char *modname, int weak)
 			return s->owner;
 	}
 
-	/* Ignore __start_* and __stop_* like kernel might. */
-	if (print_unknown && !weak
-	    && (strncmp(name, "__start_", strlen("__start_")) != 0
-		|| strncmp(name, "__stop_", strlen("__stop_")) != 0))
+	if (print_unknown && !weak)
 		warn("%s needs unknown symbol %s\n", modname, name);
 
 	return NULL;
@@ -168,6 +165,8 @@ static void load_system_map(const char *filename)
 
 	/* __this_module is magic inserted by kernel loader. */
 	add_symbol("__this_module", NULL);
+	/* On S390, this is faked up too */
+	add_symbol("_GLOBAL_OFFSET_TABLE_", NULL);
 }
 
 static struct option options[] = { { "all", 0, NULL, 'a' },
@@ -183,6 +182,8 @@ static struct option options[] = { { "all", 0, NULL, 'a' },
 				   { "unresolved-error", 0, NULL, 'u' },
 				   { "verbose", 0, NULL, 'v' },
 				   { "version", 0, NULL, 'V' },
+				   /* Obsolete, but we need to parse it. */
+				   { "config", 1, NULL, 'C' },
 				   { NULL, 0, NULL, 0 } };
 
 /* Version number or module name?  Don't assume extension. */
@@ -594,6 +595,8 @@ static struct depfile depfiles[] = {
 	{ "modules.usbmap", output_usb_table },
 	{ "modules.ccwmap", output_ccw_table },
 	{ "modules.ieee1394map", output_ieee1394_table },
+	{ "modules.isapnpmap", output_isapnp_table },
+	{ "modules.inputmap", output_input_table },
 	{ "modules.alias", output_aliases },
 	{ "modules.symbols", output_symbols },
 };
@@ -655,7 +658,7 @@ int main(int argc, char *argv[])
 	/* Don't print out any errors just yet, we might want to exec
            backwards compat version. */
 	opterr = 0;
-	while ((opt = getopt_long(argc, argv, "ab:ArehnqruvVF:", options, NULL))
+	while ((opt = getopt_long(argc, argv, "ab:ArehnqruvVF:C:", options, NULL))
 	       != -1) {
 		switch (opt) {
 		case 'a':
@@ -680,6 +683,7 @@ int main(int argc, char *argv[])
 		case 'u':
 		case 'q':
 		case 'r':
+		case 'C':
 			/* Ignored. */
 			break;
 		case 'h':
@@ -703,9 +707,10 @@ int main(int argc, char *argv[])
 		load_system_map(system_map);
 
 	/* They can specify the version naked on the command line */
-	if (optind < argc && is_version_number(argv[optind]))
+	if (optind < argc && is_version_number(argv[optind])) {
 		version = strdup(argv[optind]);
-	else {
+		optind++;
+	} else {
 		struct utsname buf;
 		uname(&buf);
 		version = strdup(buf.release);
