@@ -30,6 +30,10 @@
 
 #include "backwards_compat.c"
 
+#define streq(a,b) (strcmp((a),(b)) == 0)
+
+extern long init_module(void *, unsigned long, const char *);
+
 static void print_usage(const char *progname)
 {
 	fprintf(stderr, "Usage: %s filename [args]\n", progname);
@@ -53,23 +57,57 @@ static const char *moderror(int err)
 	}
 }
 
+static void *grab_file(const char *filename, unsigned long *size)
+{
+	unsigned int max = 16384;
+	int ret, fd;
+	void *buffer = malloc(max);
+
+	if (streq(filename, "-"))
+		fd = dup(STDIN_FILENO);
+	else
+		fd = open(filename, O_RDONLY, 0);
+
+	if (fd < 0)
+		return NULL;
+
+	*size = 0;
+	while ((ret = read(fd, buffer + *size, max - *size)) > 0) {
+		*size += ret;
+		if (*size == max)
+			buffer = realloc(buffer, max *= 2);
+	}
+	if (ret < 0) {
+		free(buffer);
+		buffer = NULL;
+	}
+	close(fd);
+	return buffer;
+}
+
 int main(int argc, char *argv[])
 {
 	unsigned int i;
-	int fd;
 	long int ret;
-	struct stat st;
 	unsigned long len;
-	void *map;
+	void *file;
 	char *filename, *options = strdup("");
 	char *progname = argv[0];
 
-	try_old_version("insmod", argv);
+	if (strstr(argv[0], "insmod.static"))
+		try_old_version("insmod.static", argv);
+	else
+		try_old_version("insmod", argv);
+
+	if (argv[1] && (streq(argv[1], "--version") || streq(argv[1], "-V"))) {
+		puts(PACKAGE " version " VERSION);
+		exit(0);
+	}
 
 	/* Ignore old options, for backwards compat. */
-	while (argv[1] && (strcmp(argv[1], "-p") == 0
-			   || strcmp(argv[1], "-s") == 0
-			   || strcmp(argv[1], "-f") == 0)) {
+	while (argv[1] && (streq(argv[1], "-p")
+			   || streq(argv[1], "-s")
+			   || streq(argv[1], "-f"))) {
 		argv++;
 		argc--;
 	}
@@ -92,25 +130,16 @@ int main(int argc, char *argv[])
 		strcat(options, " ");
 	}
 
-	fd = open(filename, O_RDONLY, 0);
-	if (fd < 0) {
-		fprintf(stderr, "Can't open '%s': %s\n",
+	file = grab_file(filename, &len);
+	if (!file) {
+		fprintf(stderr, "insmod: can't read '%s': %s\n",
 			filename, strerror(errno));
 		exit(1);
 	}
 
-	fstat(fd, &st);
-	len = st.st_size;
-	map = mmap(NULL, len, PROT_READ, MAP_SHARED, fd, 0);
-	if (map == MAP_FAILED) {
-		fprintf(stderr, "Can't map '%s': %s\n",
-			filename, strerror(errno));
-		exit(1);
-	}
-
-	ret = syscall(__NR_init_module, map, len, options);
+	ret = init_module(file, len, options);
 	if (ret != 0) {
-		fprintf(stderr, "Error inserting '%s': %li %s\n",
+		fprintf(stderr, "insmod: error inserting '%s': %li %s\n",
 			filename, ret, moderror(errno));
 		exit(1);
 	}
